import os
import os.path
import shutil
import string
import urllib
import kraLog
import kraUtils
import kraConvert
import kraQuery
from kraConstants import kraConstants
from kraPresentation import generateText, generateHtml
from kraLog import log
from kraEntry import kraEntry, loadKraEntry
from kraMetaData import kraMetaData, loadMetaData, generateCache
# backward compatibility
from cocConstants import cocConstants
from cocData import cocData, loadCocData

#
#  Prints out the configuration
#
def kraCommand_ShowConfiguration(config, runtime):
    print "\n" + str(config)    

#
#  Writes out a list of the cocoon in the html format.
#
def kraCommand_GenerateHTML(config, runtime):
    log("Generating html...")
    if runtime.outputFile == "":
       runtime.outputFile = config.destinationDir + "/" + cocConstants.COCOON_DEFAULT_HTML_FILE
    metadata = loadMetaData(config, runtime) 
    metadata = kraQuery.filterMetaData(metadata, runtime)
    temp = generateHtml(config, runtime, metadata)
    kraUtils.writeOutput(config, runtime, temp)

#
#  Writes out a list of the kraken - usually, with a query
#
def kraCommand_GenerateText(config, runtime):    
    metadata = loadMetaData(config, runtime)
    metadata = kraQuery.filterMetaData(metadata, runtime)
    temp = generateText(config, runtime, metadata)
    kraUtils.writeOutput(config, runtime, temp)

#
#  Writes out a summary of the kraken
#
def kraCommand_ShowSummary(config, runtime):    
    metadata = loadMetaData(config, runtime)
    print "\n" + str(metadata)

#
#  Creates a backup of the kraken
#
def kraCommand_BackupKraken(config, runtime):
    command = "tar cfz KRAKEN_BACKUP.tgz"    
    for a in config.krakenDirs:
        command = command + " " + a
    print "Executing command: " + command
    os.system(command)

# **************************************************
#   Annotate entry
# **************************************************
def kraCommand_AnnotateEntry(config, runtime):
    # FIXME: for the time being works only for the destination dir
    print "Annotating entry: " + runtime.annotateEntry
    entry = loadKraEntry(config, runtime.annotateEntry)
    if entry == None:
       cleanExit(config, runtime, 1, "Can not annotate entry: " + runtime.annotateEntry)
    print entry
    entry.edit()
    entry.writeRDF(config.destinationDir + runtime.annotateEntry + "/" 
          + kraConstants.FILENAME_ENTRY_RDF)
    # FIXME: the cache can be updated without regenerating.  
    generateCache(config, runtime)

    
#
#  Creates an entry
# 
def kraCommand_CreateEntry(config, runtime):
    if runtime.inputFile != "":
        runtime.inputFile = os.path.expanduser(runtime.inputFile)
        runtime.inputFile = os.path.abspath(runtime.inputFile)
        entry = createEntryForSingleFile(config, runtime)
    elif runtime.inputHttp != "":
        if not runtime.recursive:
            entry = createEntryForSingleWebpage(config, runtime)
        else:
            entry = createEntryForFullWebpage(config, runtime)
    else:
        cleanExit(config, runtime, 0, "I don't understand what do you want me to do")        
    entry.fillinEntryFromRuntime(runtime)
    entry.writeRDF(config.tempDir + "/" + runtime.entryDir + "/" + kraConstants.FILENAME_ENTRY_RDF)
    # print entry
    log("Kraken entry " + runtime.entryDir + " successfully created")
    kraUtils.moveEntryToDestinationDir(config, runtime)
    # FIXME: the cache can be updated without regenerating
    generateCache(config, runtime)

#
#  Add a directory
# 
def kraCommand_AddDirectory(config, runtime):
    runtime.directoryToAdd = os.path.expanduser(runtime.directoryToAdd)
    dir = kraUtils.checkKrakenDir(runtime.directoryToAdd)
    if dir != None:
       config.krakenDirs.append(dir)
       config.dirty = 1
       log("Directory " + runtime.directoryToAdd + " succesfully appended to the kraken.")
    generateCache(config, runtime)

#
#  Removes a directory
# 
def kraCommand_RemoveDirectory(config, runtime):
    if not (runtime.directoryToRemove in config.krakenDirs):
        log("Directory "+ runtime.directoryToRemove + " is not a member of the kraken, cannot remove", kraLog.WARNING)
        return
    config.krakenDirs.remove(runtime.directoryToRemove)
    config.dirty = 1
    log("Directory " + runtime.directoryToRemove + " succesfully removed from the kraken.")
    generateCache(config, runtime)
    
#
#  Embrace entries
#      
def kraCommand_EmbraceEntries(config, runtime):
    for kd in config.krakenDirs:
        for x in os.listdir(kd):            
           if not os.path.isdir(kd + "/" + x):
               continue
           if os.path.exists(kd + "/" + x + "/"+kraConstants.FILENAME_ENTRY_RDF):
               continue
           dirToEmbrace = kd + "/" + x + "/"
           __embraceSingleEntry(dirToEmbrace, dirName, config, runtime)
    # FIXME: the cache can be updated without regenerating
    generateCache(config, runtime)

           
def __embraceSingleEntry(dirToEmbrace, dirName, config, runtime):
    files = os.listdir(dirToEmbrace)
    log("\n\nEmbracing directory: " + dirName)
    log("Contents of directory:\n" + str(files) + "\n")
    if not kraUtils.confirm("Should I proceed with embracing?"):
        return
    entry = kraEntry()
    entry.directoryName = dirName
    if (len(files) <= 3):
        entry.localFileNames = files 
    entry.edit()
    entry.writeRDF(dirToEmbrace+ "/" + kraConstants.FILENAME_ENTRY_RDF)
    
    
# ***************************************************************
#  Create the entry for a single file from the local filesystem
# ***************************************************************
def createEntryForSingleFile(config, runtime):
    # Fixme: check if the file exists.
    if runtime.inputFile == "":
        cleanExit(config, 1, "It is required to have a file")
    if not os.path.exists(runtime.inputFile):
        cleanExit(config, 1, runtime.inputfile + " does not exists!!!")
    kraUtils.createEntry(config,runtime)    
    shutil.copy2(runtime.inputFile, config.tempDir + "/" + runtime.entryDir)
    entry = kraEntry()
    entry.originalURIs = [runtime.inputFile]    
    entry.localFileNames = [os.path.split(runtime.inputFile)[1]]
    log(config, "Kraken entry " + runtime.entryDir + " successfully created")
    return entry
    
  
# ***********************************************************
#  Create a kraken entry for a single webpage
# ***********************************************************
def createEntryForSingleWebpage(config, runtime):
    kraUtils.createEntry(config, runtime)
    targetdir = config.tempDir+"/"+runtime.entryDir
    if os.name == "nt":
       targetdir = "\"" + string.replace(targetdir,"/","\\\\") + "\""
    command = (kraConstants.COMMAND_DOWNLOAD_SINGLE_FILE + runtime.inputHttp +
          " --directory-prefix=" + targetdir)
    log("Executing: " + command)
    os.system(command)
    # now, create the cocoon file
    entry = kraEntry()
    entry.originalURIs = [runtime.inputHttp]
    # get the http filename, simply picking the first file in the directory    
    for x in os.listdir(config.tempDir+"/"+runtime.entryDir):
       if not os.path.isdir(x):
           entry.localFileNames = [x]
           #print entry.localFileNames
    return entry;

# **********************************************************
#  Create the cocoon for a full webpage
# ***********************************************************
def createEntryForFullWebpage(config, runtime):
    if os.name == "nt":
        # targetdir = "\"" + string.replace(targetdir,"/","\\\\") + "\""
        print "This command does not work correctly in Windows, because of some file"
        print "name mapping issues. It is a wget problem, but until I find a solution"
        print "I have disabled it"
        os._exit(0)       
    kraUtils.createEntry(config, runtime)
    targetdir = config.tempDir+"/"+runtime.entryDir
    print targetdir      
    command = (kraConstants.COMMAND_DOWNLOAD_FULLPAGE + runtime.inputHttp +
      " --directory-prefix="+targetdir)
    print command
    os.system(command)
    # now, create the cocoon file
    entry = kraEntry()
    entry.originalURIs = [runtime.inputHttp]
    temp = runtime.inputHttp[7:] # got rid of http://
    #if temp[-1] != "/": temp += "/"
    # get the http filename, simply picking the first file in the directory
    resultFile = config.tempDir+"/"+runtime.entryDir + "/" + temp
    print "Resultfile: " + resultFile
    if os.path.isdir(resultFile):    
       listdir = os.listdir(resultFile)
       for priv in ("index.html", "index.htm", os.path.split(runtime.inputFile)[1]):
          if priv in listdir:
             entry.localFileNames = [temp + priv] 
             log("Filename is:" + entry.localFileNames[0])
          else:
             log("Could not determine the local file name")
    elif os.path.exists(resultFile):
        entry.localFileNames = [temp]
    else:
        log("Can not identify the main file"+temp,kraLog.WARNING)
    return entry

    

