# **************************************************
#  The configuration class
#
# **************************************************

import pickle
import os
import os.path
from kraConstants import kraConstants
from kraLog import log
import kraLog
import kraUtils2

class kraConfig:            
    def __init__(self):
        # configuration - should normally persist accross calls
        # the destination directory (this is where new entries are added)
        self.destinationDir = os.path.expanduser(kraConstants.DEFAULT_DESTINATION_DIR)
        # the directories which hold the kraken items. These are searched for
        # queries, archived etc
        self.krakenDirs = [self.destinationDir]
        # the staging dir for entry creation
        self.tempDir = os.path.expanduser(kraConstants.DEFAULT_STAGING_DIR)
        # operating mode 0 - command line, 1 - interactive TO COME: 2 - gui
        self.interactive = 0
        # logger verbosity levels
        self.verbosity = 5 # range 1..5
        # do I need it?
        self.dirty = 1
    
    # experiments in printing etc. 
    def __str__(self):
        x = "Kraken configuration:"
        x = x + "\n\tDestination dir: " + self.destinationDir 
        x = x + "\n\tKraken directories:"
        for d in self.krakenDirs:
            x = x + "\n\t\t" + d 
        x = x + "\n\tStaging dir: " + self.tempDir
        x = x + "\n\tOperating mode: "
        if (self.interactive):
            x = x + "interactive"
        else:
            x = x + "command line"
        x = x + "\n\tVerbosity level (1 lowest, 5 highest): " + str(self.verbosity)
        return x + "\n"
    
         

               
# ****************************************************
#  checks the staging dir and creates it if necessary
# FIXME: should also check if it is empty
# ****************************************************  
    def checkStagingDir(self):
       if self.tempDir[-1] != "/":
           self.tempDir += "/"
           self.dirty = 1
       if os.path.isdir(self.tempDir) and len(os.listdir(self.tempDir)) != 0:
            log("The staging directory " + self.tempDir + " is not empty. "
               +"Can not proceed, please clean it manually.", kraLog.ERROR)
            os._exit(1)
       val = kraUtils2.checkAndAttemptToCreateDirectory(self.tempDir, 
                 "Staging directory")
       if not val:
            log("Can not proceed without a staging directory", kraLog.ERROR)
            os._exit(1)       
       return val
         
              

# 
# get config file - will allow later for customization
# first, try to load from the ~/.cocoonrc
# 
def _getConfigFile():
    return kraConstants.CONFIG_FILE

#
# creates the config file, either by loading it, or creating a new one
# 
def createConfig():
    try:
        filename = os.path.expanduser(_getConfigFile())
        FILE = open(filename, "r")
        coc = pickle.load(FILE)
    except:
        # if not successful, create
        log("Could not read config file from "+_getConfigFile()+". Creating default.", kraLog.WARNING)
        coc = kraConfig()
    # The directories need to end with "/"
    if coc.checkStagingDir():
      return coc
    else:
      return 0 # should exit

#
# saves the config file
#    
def saveConfig(config):
    if config.dirty:
        config.dirty = 0
        filename = os.path.expanduser(kraConstants.CONFIG_FILE)
        FILE = open(filename, "w")
        pickle.dump(config, FILE)
        FILE.close()
