# **************************************************
#
#  The kraken data - this is the format for the 
#  kraken version 0.5 files.
#
# **************************************************

import pickle
import time
import sys
import os.path
import string
import urllib
import strptime
import kraUtils
from rdflib.URIRef import URIRef
from rdflib.Literal import Literal
from rdflib.BNode import BNode
from rdflib.Identifier import Identifier
from rdflib.Namespace import Namespace
from rdflib.constants import TYPE, ABOUT

# Import RDFLib's default TripleStore implementation.
from rdflib.TripleStore import TripleStore

import kraLog
from kraLog import log
from kraConstants import kraConstants
import random

KRAKEN_ENTRY = Namespace("http://localhost/krakenEntrySchema/")

class kraEntry:            

    def __init__(self):
       # the label. This is a fictive URL: kraken://
       self.label = "kraken://" + str(random.random())
       # the name of the directory inside the kraken. This is considered
       # to be mildly significant
       self.directoryName = ""
       # the title of the enty
       self.title = ""
       # short textual description of the entry
       self.description = ""
       # the date when it was created
       self.date = time.localtime()
       # the name of top level files. If there is more than one, it should
       # as a continuation. There can be extra files (icons etc) which do not
       # appear here
       self.localFileNames = []
       # the original URLs of the entries
       self.originalURIs = []
       # the keywords
       self.keywords = [] # comma separated
       # the size of the entry - what am I going to use this for?
       self.size = 0
       
       # Non-persistent data (is not serialized in RDF)
       
       # the kraken dir name (for multiple directories)
       self.krakenDirName = "" 
       
        
    #def writeToFile(self, filename):
    #  FILE = open(filename, "w")
    #  pickle.dump(self, FILE)
    #  FILE.close()          
     
     # **************************************************
     #   fillin data from runtime
     #   this refers to the annotation data, not the original stuff, etc
     # **************************************************
    def fillinEntryFromRuntime(self, runtime, force = 0): 
        self.directoryName = runtime.entryDir 
        if force or runtime.title != "":
            self.title = runtime.title
        if force or runtime.description != "":     
            self.description = runtime.description
        if force or runtime.keywords != []:  
            self.keywords = runtime.keywords
        if force or runtime.date != 0:
            self.date = runtime.date

    #
    #  Command line based editing of an entry - mainly for annotation, etc
    #            
    def edit(self):
        print "Label = " + self.label
        self.title = kraUtils.inputWithDefault("Title", self.title)
        self.description = kraUtils.inputWithDefault("Description", self.description)
        # FIXME: keyword validation, etc
        self.keywords = kraUtils.inputSequenceWithDefault("Keywords", self.keywords)
        # FIXME: edit the date
        temp = kraUtils.inputWithDefault("Date", time.strftime("%a, %d %b %Y %H:%M:%S +0000", self.date))
        self.date = strptime.strptime(temp, "%a, %d %b %Y %H:%M:%S +0000")
            
    def _createStore(self):
       # KRAKEN_ENTRY = Namespace("krakenEntrySchema.rdf")
       KRAKEN_ENTRY = Namespace("http://localhost/krakenEntrySchema/")
       store = TripleStore()
       store.prefix_mapping("dc", "http://http://purl.org/dc/elements/1.1/")
       store.prefix_mapping("foaf", "http://xmlns.com/foaf/0.1/")
       store.prefix_mapping("krakenEntry", "http://localhost/krakenEntrySchema/")
       # store.load("krakenEntrySchema.rdf")
       return store

    def __addProp(self, propname):
        value = self.__dict__[propname]        
        self.store.add((self.entry, KRAKEN_ENTRY[propname], Literal(value)))
   
    # def __getProp(self, propname):       
        
    def writeRDF(self, fileName):
       self.store = self._createStore()
       self.entry = URIRef("http://localhost/theEntry")       
       self.__addProp("label")
       self.__addProp("directoryName")
       self.__addProp("title")
       self.__addProp("description")
       self.__addProp("date")
       self.__addProp("localFileNames")
       self.__addProp("originalURIs")
       self.__addProp("keywords")
       self.__addProp("size")
       # print "Saving to: "+fileName
       self.store.save(fileName)
       self.store = None
       
    def loadRDF(self, fileName):
       urlName = urllib.pathname2url(fileName)
       KRAKEN_ENTRY = Namespace("http://localhost/krakenEntrySchema/")
       store = self._createStore()
       if (not os.path.exists(fileName)):
          print fileName + "does not exist" 
          raise 0
       store.load(urlName)
       resource = store[URIRef("http://localhost/theEntry")]
       # and now, fill in the kraEntry from the loaded stuff.
       self.label = str( resource[KRAKEN_ENTRY["label"]] )
       self.directoryName = str( resource[KRAKEN_ENTRY["directoryName"]] )
       self.title = str( resource[KRAKEN_ENTRY["title"]] )
       self.description = str( resource[KRAKEN_ENTRY["description"]] )
       self.localFileNames = eval (resource[KRAKEN_ENTRY["localFileNames"]] )
       self.originalURIs = eval (resource[KRAKEN_ENTRY["originalURIs"]])
       self.size = resource[KRAKEN_ENTRY["size"]]
       self.keywords = eval( resource[KRAKEN_ENTRY["keywords"]] )
       self.date = eval (resource[KRAKEN_ENTRY["date"]] )
       
      
    def __str__(self): 
        x = "kraEntry:\n"
        x = x + "\tlabel = " + self.label
        x = x + "\n\ttitle = " + self.title
        x = x + "\n\tdescription = " + self.description
        x = x + "\n\tdate = " + time.strftime("%a, %d %b %Y %H:%M:%S +0000", self.date)
        x = x + "\n\tlocalFileNames = " + str(self.localFileNames)
        x = x + "\n\toriginalURIs = " + str(self.originalURIs)
        x = x + "\n\tkeywords = " + str(self.keywords)
        x = x + "\n\tsize = " + str(self.size)
        return x

#
#  Loading an entry from the file. FIXME: this is not quity good because it 
#  works only for the destination dir.
#        
def loadKraEntry(config, fileName):
  try:    
    temp = kraEntry()
    fileToOpen = fileName + "/"+kraConstants.FILENAME_ENTRY_RDF
    temp.loadRDF(fileToOpen)
    return temp
  except:
    print "Unexpected error:", sys.exc_info()
    log("Can not open file: "+fileToOpen, kraLog.WARNING)
    return None
    
    
def test():
    k = kraEntry()
    k.label = "kraken://ss"
    k.title = "Excite homepage"
    k.description = "The homepage on excite"
    k.localFileNames = ["index.html"]
    k.originalURIs = ["http://www.excite.com"]
    k.keywords = ("Test", "Internet")
    print k

    k.writeRDF("testentry.rdf")
    d = kraEntry()   
    d.loadRDF("testentry.rdf")
    print d 

    FILE = open("testentry.dat", "w")
    pickle.dump(k, FILE)
    FILE.close()
    print "Test2 done"          

#test2()

