import os
import string
import shutil
import copy
from kraLog import log
from kraConfig import kraConfig, createConfig, saveConfig
import kraUtils2
from kraConstants import kraConstants

# this is fixing the problem that in windows we don't have readline
if os.name == 'posix': 
    import readline
    haveReadline = 1
else:
    haveReadline = 0


# **************************************************
#   Helper function: writing out outputs. 
# **************************************************
def writeOutput(config, runtime, temp):
  if runtime.outputFile == "":
     print temp # fixme, write in output file
  else:
     log("Writing to output file:" + runtime.outputFile)
     OUTPUTFILE = open(runtime.outputFile, "w")
     OUTPUTFILE.write(temp)
     OUTPUTFILE.close()
     log("Done.")

# *********************************************************
#   Function for exit, error message etc. 
# **********************************************************
def cleanExit(config, runtime, exitValue = 0, message = ""):
    if message != "":
        print message
    if exitValue == 0:
        saveConfig(config)
    os._exit(exitValue)    
     
# **************************************************
#
#   Helper function. A relatively sophisticated stuff 
#   to allow initilizing the readline stuff with some data
#
# **************************************************        
def inputWithDefault(varname, initvalue):
        if haveReadline:          
            def __defaultStartupHook():
                readline.insert_text(__currentDefault)        
            __currentDefault = initvalue        
            readline.set_startup_hook(__defaultStartupHook)
            temp = raw_input(varname + " = ")
            readline.set_startup_hook()
        else:
            print varname + " = " + initvalue + "\n\t(<ENTER> to keep)"
            temp = raw_input(varname + " = ")
            if temp == "":
                return initvalue
        return temp
        
# **************************************************
#
#   Helper function. A relatively sophisticated stuff 
#   to allow initilizing the readline stuff with some data
#
# **************************************************    
def inputSequenceWithDefault(varname, initvalue, nsstringlist = []):
    stringlist = copy.copy(nsstringlist)
    stringlist.sort()
    while 1:
        done = 1
        tmp = ""
        for a in initvalue:
            tmp = tmp + a + ","
        tmp = tmp[:-1]
        tmp = inputWithDefault(varname, tmp)
        temp = string.split(tmp, ",")
        temp = map(string.strip, temp)
        if stringlist != []:
          for a in temp:
             if a not in stringlist:
                 print a + " is not a keyword."
                 print "The list of suggested choices are: " + str(stringlist)
                 if not confirm("Do you want to proceed (Y) or continue editing (N) ?"):
                     done = 0
                     break; 
        if done: return temp
    

def confirm(question):
      reply = raw_input(question + " (Y/N) ")
      if reply not in "yY":
          return 0
      return 1            
        
# ****************************************************
#  checks a kraken  directory and creates it if necessary
# ****************************************************    
def checkKrakenDir(directory):
       if directory[-1] != "/":
           directory += "/"
           # self.dirty = 1       
       val = kraUtils2.checkAndAttemptToCreateDirectory(directory, "Destination directory")
       if not val:
          return None
       if os.path.exists(directory + kraConstants.KRAKEN_DIR_INDICATOR_FILE):
          return directory
       # check if the directory is empty
       FILE = open(directory + kraConstants.KRAKEN_DIR_INDICATOR_FILE, "w")
       FILE.write("KRAKEN")
       FILE.close()
       print "Kraken directory:" + directory + " succesfully initialized"
       return directory
        
# *********************************************************
#   Returns the largest cocoon in the destination directory
# **********************************************************
def createEntryName(config, runtime):
    if not checkKrakenDir(config.destinationDir):
        cleanExit(config, runtime, 1, "Do not have a working destination directory, leaving.")
    currentMax = 0
    for x in os.listdir(config.destinationDir):
        if (os.path.isdir(config.destinationDir + "/" + x) 
           and x[0:len(kraConstants.DIRECTORY_PREFIX)] == kraConstants.DIRECTORY_PREFIX):
            count = int(x[len(kraConstants.DIRECTORY_PREFIX):])
            if count > currentMax: currentMax = count
    return kraConstants.DIRECTORY_PREFIX+string.zfill(currentMax+1,5)
  
        
# **********************************************************
#  Create the entry directory
# ***********************************************************
def createEntry(config, runtime):
    runtime.entryDir = createEntryName(config, runtime)
    os.mkdir(config.tempDir + "/" + runtime.entryDir)


# **************************************************
#   Move the cocoon to the destination directory
# **************************************************
def moveEntryToDestinationDir(config, runtime):
    shutil.copytree(config.tempDir + "/" + runtime.entryDir, config.destinationDir + runtime.entryDir)
    shutil.rmtree(config.tempDir + "/" + runtime.entryDir)
        
