#!/usr/bin/env python

import string
import time
import optik

from kraConfig import createConfig
from kraRuntime import kraRuntime
from kraConstants import kraConstants
from kraLog import log, setLogConfig
from kraCommands import *
from kraUtils import cleanExit
import kraken.Version

    
# **************************************************
#   Process the arguments
# **************************************************
def processArguments():
#  global conf # fixme: do we need this?
  parser = optik.OptionParser()
  
  # commands
  parser.add_option("--backup-kraken", dest="theCommand", action="store_const",
                    const=kraRuntime.BACKUP_KRAKEN,
                    help="shows the configuration")                    
  
  # configuration options
  parser.add_option("--tempdir", dest="tempDir",
                    help="temporary, staging directory (persistent)", metavar="FILE")
  parser.add_option("--destdir", dest="destinationDir",
                    help="destination directory (persistent)", metavar="FILE" )
  parser.add_option("-i","--interactive", dest="interactive", action="store_true",
                    help="interactive entry (persistent)")
  parser.add_option("--non-interactive", dest="interactive", action="store_false",
                    help="non interactive entry (persistent)")
  parser.add_option("--inputfile", dest="inputFile",
                    help="the inputfile", metavar="FILE")
  parser.add_option("--inputhttp", dest="inputHttp",
                    help="the input webaddress", metavar="SIZE")
  parser.add_option("-r", "--recursive", dest="recursive", action="store_true",
                    help="recursive download of the webpage")
  parser.add_option("--annotate-entry", dest="annotateEntry",
                    help="annotate the specified entry", metavar="ENTRY")
  parser.add_option("--title", dest="title",
                    help="the title of the page", metavar="TITLE")
  parser.add_option("--description", dest="description",
                    help="the description of the page", metavar="DESCRIPTION")
  parser.add_option("--categories", dest="tmpKeywords",
                    help="comma separated list of keywords", metavar="KEY1,KEY2...")
  parser.add_option("--date", dest="tmpDate",
                    help="date of file (mm/dd/yy)", metavar="DATE")
  parser.add_option("--force-regenerate-cache", dest="forceRegenerateCache",
                    action="store_true",
                    help="forces the regeneration of the cache")
                    
  # Commands for listing, generating and querying
                    
  parser.add_option("--generate-text", dest="theCommand", action="store_const",
                    const=kraRuntime.GENERATE_TEXT,
                    help="generate a text listing")
  parser.add_option("-g", "--generate-html", dest="theCommand", action="store_const",
                    const=kraRuntime.GENERATE_HTML,
                    help="generate a html page")                    
  parser.add_option("--show-config", dest="theCommand", action="store_const",
                    const=kraRuntime.SHOW_CONFIGURATION,
                    help="shows the configuration")
  parser.add_option("--show-summary", dest="theCommand", action="store_const",
                    const=kraRuntime.SHOW_SUMMARY,
                    help="shows the configuration")                    
  parser.add_option("--add-directory", dest="directoryToAdd", action="store",
                    help="add a directory to the kraken")
  parser.add_option("--remove-directory", dest="directoryToRemove", action="store",
                    help="remove a directory from the kraken")
  parser.add_option("-e", "--embrace-entries", dest="theCommand", action="store_const",
                    const=kraRuntime.EMBRACE_ENTRIES,
                    help="embrace entries from directories.")

  # query parameters                    
  parser.add_option("--select-keywords-all", dest="selectKeywordsAll",
                    help="Select instances which have all the keywords", metavar="KEY1,KEY2" )
  parser.add_option("--select-keywords-any", dest="selectKeywordsAny",
                    help="Select instances which have any of the keywords", metavar="KEY1,KEY2" )
  parser.add_option("--outputfile", dest="outputFile",
                    help="the outputfile for generate commands", metavar="FILE")
                                        
  (options, runtime.arguments) = parser.parse_args(values=runtime)
  
def postProcessArguments():  
  # post process the options
  # split the comma separated categories into a list
  if runtime.tmpKeywords != "":
      runtime.keywords = string.split(runtime.tmpKeywords, ",")
      # print "Categories list: " + str(runtime.categories)
  # directory to add / remove
  if runtime.directoryToAdd != "":
      runtime.theCommand = kraRuntime.ADD_DIRECTORY
  if runtime.directoryToRemove != "":
      runtime.theCommand = kraRuntime.REMOVE_DIRECTORY
  if runtime.annotateEntry != "":
      runtime.theCommand = kraRuntime.ANNOTATE_ENTRY
  
  # process the date, if any
  if runtime.tmpDate != "":
      try:
        runtime.date = time.strptime(runtime.tmpDate, "%m/%d/%y")
      except:
        cleanExit(config, runtime, 1,
           "Error: " + runtime.tmpDate + " is not a valid date. Use mm/dd/yy format!")
  
  # process the runtime argument            
  if len(runtime.arguments) > 1:
      cleanExit(config, runtime, 1, "Only one runtime argument is allowed")
  if len(runtime.arguments) == 1:
      runtime.theCommand = kraRuntime.CREATE_ENTRY
      temp = runtime.arguments[0]
      if (temp[0:7] == "http://"):
          runtime.inputHttp = temp
      else:
          runtime.inputFile = temp
  
  # Copying from runtime to config
  if "tempDir" in dir(runtime):
      config.tempDir = runtime.tempDir
      config.dirty = 1
  if "destinationDir" in dir(runtime):
      config.destinationDir = runtime.destinationDir
      config.dirty = 1
  if runtime.interactive != config.interactive:
      config.interactive = runtime.interactive
      config.dirty = 1

  

# **************************************************
#
#   The main program
#
# **************************************************

print "Kraken " + kraken.Version.versionNumber + " of " + kraken.Version.date + " (c) Lotzi Boloni"
runtime = kraRuntime()
processArguments()
config = createConfig()
setLogConfig(config)
runtime.interactive = config.interactive
postProcessArguments()


if (config.interactive and 
     ( (runtime.inputHttp != "" or runtime.inputFile != "") 
       or runtime.annotateEntry)):
    runtime.interactiveRead(config)

# assume that the command is CreateEntry
if runtime.theCommand == kraRuntime.CREATE_ENTRY:
    kraCommand_CreateEntry(config, runtime) 
    cleanExit(config, runtime)
      
if runtime.theCommand == kraRuntime.ANNOTATE_ENTRY:
    kraCommand_AnnotateEntry(config, runtime)
    cleanExit(config, runtime, 0)

if runtime.theCommand == kraRuntime.GENERATE_TEXT:
    kraCommand_GenerateText(config, runtime)
    cleanExit(config, runtime, 0)  
    
if runtime.theCommand == kraRuntime.GENERATE_HTML:
    kraCommand_GenerateHTML(config, runtime)
    cleanExit(config, runtime, 0)
 
if runtime.theCommand == kraRuntime.ADD_DIRECTORY:
    kraCommand_AddDirectory(config, runtime)
    cleanExit(config, runtime, 0)
    
if runtime.theCommand == kraRuntime.REMOVE_DIRECTORY:
    kraCommand_RemoveDirectory(config, runtime)
    cleanExit(config, runtime, 0)
    
if runtime.theCommand == kraRuntime.SHOW_CONFIGURATION:
    kraCommand_ShowConfiguration(config, runtime)
    cleanExit(config, runtime, 0)
    
if runtime.theCommand == kraRuntime.SHOW_SUMMARY:
    kraCommand_ShowSummary(config, runtime)
    cleanExit(config, runtime, 0)

if runtime.theCommand == kraRuntime.BACKUP_KRAKEN:
    kraCommand_BackupKraken(config, runtime)
    cleanExit(config, runtime, 0)
    
if runtime.theCommand == kraRuntime.EMBRACE_ENTRIES:
    kraCommand_EmbraceEntries(config, runtime)
    cleanExit(config, runtime, 0)
    
print "Nothing to do: " + str(runtime.theCommand)
cleanExit(config, runtime, 0)

